@@ -80,6 +80,7 @@ tokenize_spaces_punct <- function(x){
 #'  \item{text: }{The provided character string of either a or b}
 #'  \item{tokens: }{A character vector of the tokenised texts of a or b}
 #'  \item{n: }{The length of \code{tokens}}
+#'  \item{similarity: }{The similarity to a calculated as the Smith-Waterman local alignment score / (the number of letters/words in the a or b text times the match weight)}
 #'  \item{alignment: }{A list with the following elements}
 #'    \itemize{
 #'    \item{text: }{The aligned text from either a or b where gaps/mismatches are filled up with the \code{edit_mark} symbol}
@@ -201,6 +202,7 @@ smith_waterman <- function(a, b,
                       a = list(text = a,
                                tokens = original_a,
                                n = length(standardised_a),
+                               similarity = 0,
                                alignment = list(text = character(),
                                                 tokens = character(),
                                                 n = 0L,
@@ -210,6 +212,7 @@ smith_waterman <- function(a, b,
                       b = list(text = b,
                                tokens = original_b,
                                n = length(standardised_b),
+                               similarity = 0,
                                alignment = list(text = character(),
                                                 tokens = character(),
                                                 n = 0L,
@@ -236,6 +239,11 @@ smith_waterman <- function(a, b,
                            row_i = row_i, col_i = col_i, max_match = max_match,
                            edit_mark = edit_mark)
   }
+  # fix issue for alignment of 1 letter only
+  if(length(path$b$sequence) == 1) 
+    path$b$from <- path$b$to 
+  if(length(path$a$sequence) == 1) 
+    path$a$from <- path$a$to
   
   # similarity: alignment score / score in case of perfect match (note does not work when user provides his own similarity function)
   similarity <- alignment_score / (min(length(standardised_a), length(standardised_b)) * match)
@@ -249,6 +257,7 @@ smith_waterman <- function(a, b,
                     a = list(text = a,
                              tokens = original_a,
                              n = length(standardised_a),
+                             similarity = alignment_score / (length(standardised_a) * match),
                              alignment = list(text = paste(path$a$sequence, collapse = collapse),
                                               tokens = path$a$sequence,
                                               n = length(path$a$sequence),
@@ -258,6 +267,7 @@ smith_waterman <- function(a, b,
                     b = list(text = b,
                              tokens = original_b,
                              n = length(standardised_b),
+                             similarity = alignment_score / (length(standardised_b) * match),
                              alignment = list(text = paste(path$b$sequence, collapse = collapse),
                                               tokens = path$b$sequence,
                                               n = length(path$b$sequence),
@@ -289,6 +299,8 @@ alignment_path <- function(m, original_a, original_b, row_i, col_i, max_match, e
   gaps_a <- 0L
   gaps_b <- 0L
   mismatches <- 0L
+  which_start_a <- 0;
+  which_start_b <- 0;
   
   # Begin moving up, left, or diagonally within the matrix till we hit a zero
   while (m[row_i - 1, col_i - 1] != 0) {
@@ -409,12 +421,14 @@ as.data.frame.smith_waterman <- function(x, ...){
              mismatches = x$mismatches,
              a_n = x$a$n,
              a_aligned = a_aligned, 
+             a_similarity = x$a$similarity,
              a_gaps = x$a$alignment$gaps,
              a_from = a_from,
              a_to = a_to,
              a_fromto = I(a_fromto),
              b_n = x$b$n,
              b_aligned = b_aligned, 
+             b_similarity = x$b$similarity,
              b_gaps = x$b$alignment$gaps,
              b_from = b_from,
              b_to = b_to, 
@@ -424,3 +438,76 @@ as.data.frame.smith_waterman <- function(x, ...){
 
 
 
+
+#' @title Perform multiple alignments using Smith-Waterman
+#' @description Utility function to perform all pairwise combinations of alignments between text.
+#' @param a a data.frame with columns doc_id and text. Or a character vector where the names of the character vector respresent a doc_id and the character vector corresponds to the text.
+#' @param b a data.frame with columns doc_id and text. Or a character vector where the names of the character vector respresent a doc_id and the character vector corresponds to the text.
+#' @param FUN a function to apply on an object of class \code{smith_waterman} which has done the pairwise alignment. 
+#' Defaults to \code{identity}. Other options are as.data.frame or your own function. See the examples.
+#' @param ... other arguments passed on to \code{\link{smith_waterman}}
+#' @return a list of pairwise Smith-Waterman comparisons after which the FUN argument is applied on all of these pairwise alignments.
+#' The output of the result of FUN is enriched by adding a list element 
+#' a_doc_id and b_doc_id which correspond to the doc_id's provided in \code{a} and \code{b} and which can be used
+#' in order to identify the match.
+#' @seealso \code{\link{smith_waterman}}
+#' @export
+#' @examples
+#' x <- data.frame(doc_id = c(1, 2),
+#'                 text = c("This is some text", "Another set of texts."),
+#'                 stringsAsFactors = FALSE)
+#' y <- data.frame(doc_id = c(1, 2, 3),
+#'                 text = c("were as some thing", "else, another set", NA_character_),
+#'                 stringsAsFactors = FALSE)
+#' alignments <- smith_waterman_pairwise(x, y)
+#' alignments
+#' alignments <- smith_waterman_pairwise(x, y, FUN = as.data.frame)
+#' do.call(rbind, alignments)
+#' alignments <- smith_waterman_pairwise(x, y, 
+#'                                       FUN = function(x) list(sim = x$similarity))
+#' do.call(rbind, alignments)
+#' 
+#' x <- c("1" = "This is some text", "2" = "Another set of texts.")
+#' y <- c("1" = "were as some thing", "2" = "else, another set", "3" = NA_character_)
+#' alignments <- smith_waterman_pairwise(x, y)
+smith_waterman_pairwise <- function(a, b, FUN = identity, ...){
+  as_tif <- function(x){
+    if(is.character(x) | is.factor(x)){
+      if(is.null(names(x))){
+        x <- data.frame(doc_id = seq_along(x), text = as.character(x), stringsAsFactors = FALSE)
+      }else{
+        x <- data.frame(doc_id = names(x), text = as.character(x), stringsAsFactors = FALSE)
+      }
+    }
+    x
+  }
+  set_names <- function(object, objectnames){
+    names(object) <- objectnames
+    object
+  }
+  
+  a <- as_tif(a)
+  b <- as_tif(b)
+  stopifnot(is.data.frame(a) & is.data.frame(b))
+  stopifnot(all(c("doc_id", "text") %in% colnames(a)))
+  stopifnot(all(c("doc_id", "text") %in% colnames(b)))
+  a <- a[, c("doc_id", "text"), drop = FALSE]
+  b <- b[, c("doc_id", "text"), drop = FALSE]
+  
+  combinations <- merge(a, b, by = character(), suffixes = c(".a", ".b"))
+  x <- mapply(a = set_names(combinations$text.a, combinations$doc_id.a), 
+              b = set_names(combinations$text.b, combinations$doc_id.b), 
+              FUN = function(a, b, ...){
+                alignment <- smith_waterman(a = a, b = b, ...)
+                alignment
+              }, ..., SIMPLIFY = FALSE, USE.NAMES = FALSE)
+  x <- lapply(x, FUN = FUN)
+  x <- mapply(x, 
+              a = combinations$doc_id.a, b = combinations$doc_id.b, 
+              FUN = function(x, a, b, ...){
+                x$a_doc_id <- a
+                x$b_doc_id <- b
+                x
+              }, SIMPLIFY = FALSE, USE.NAMES = FALSE)
+  x
+}
\ No newline at end of file
